Attribute

Enum Attribute 

pub enum Attribute {
    Definition {
        name: Name,
        att_type: AttType,
        default_decl: DefaultDecl,
        source: EntitySource,
    },
    Reference(Reference),
    Instance {
        name: Name,
        value: AttributeValue,
    },
    Required,
    Implied,
    Namespace {
        prefix: Prefix,
        uri: AttributeValue,
    },
}

Variants



Definition

Fields

name: Name
att_type: AttType
default_decl: DefaultDecl


Reference(Reference)



Instance

Fields

name: Name


Required



Implied



Namespace

Fields

prefix: Prefix

Implementations

Source

impl Attribute

pub fn new(name: &str, value: &str) -> Self

Useful for creating an attribute to match against in parsing specific tags.

See the ‘parse_element_with_specific_attribute_value’ example for more information.

Future use in writing XML files.

Note: this will not create attribute names that have prefixes

use nom_xml::attribute::Attribute;
let attr = Attribute::new("name","value");
Examples found in repository?
examples/parse_element_with_specific_attribute_value.rs (line 14)
8fn main() -> Result<(), Box<dyn std::error::Error>> {
9    let mut file = File::open("examples/TheExpanseSeries.xml")?;
10    let data = read_file(&mut file)?;
11    let (_, doc) = Document::parse_element_by_tag_name(
12        &data,
13        "book",
14        &Some(vec![Attribute::new("isbn", "978-0316332910")]),
15    )?;
16    println!("{doc:?}");
17    Ok(())
18}
pub fn parse_definition( input: &str, entity_references: Rc<RefCell<HashMap<(Name, EntitySource), EntityValue>>>, entity_source: EntitySource, ) -> IResult<&str, Attribute>

pub fn parse_qualified_definition( input: &str, entity_references: Rc<RefCell<HashMap<(Name, EntitySource), EntityValue>>>, entity_source: EntitySource, ) -> IResult<&str, Attribute>

pub fn parse_attvalue( input: &str, entity_references: Rc<RefCell<HashMap<(Name, EntitySource), EntityValue>>>, entity_source: EntitySource, ) -> IResult<&str, AttributeValue>

pub fn parse_attribute( input: &str, entity_references: Rc<RefCell<HashMap<(Name, EntitySource), EntityValue>>>, entity_source: EntitySource, ) -> IResult<&str, Attribute>